%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul
% Date: 31. Januar 2012
% Revision: -
%**************************************************************************

function [out]=constParamClosedLoopDistStepRespFlexShaft(Ts,decRecData)
%Input:
%  Ts = sampling time [s]
%  decRecData = decimation recording data
%Output:
%  out = constant parameter

%Step Sequenze
%-------------
u2=2/3*[0.0; 0.0; 1.2; -1.2; 0.0;...
    0.0; 1.2; -1.2; 0.0;... %reference current motor 2 [A]
    0.0; 1.2; -1.2; 0.0; 0.0]; 
r1=[0.0; 100.0; 100.0; 100.0; 100.0;...
    0.0; 0.0; 0.0; 0.0;... %reference velocity motor 1 [rad/s]
    -100.0; -100.0; -100.0; -100.0; 0.0];
Nu2=length(u2);
Nr1=length(r1);
if (Nu2 ~= Nr1)
    string1='vector u2 and r1 (closed loop disturbance step response)';
    string2=' must have the same length';
    error([string1 string2]);
end
to=2.0; %first step time [s]
ts=17.0; %max settle time one step [s]
t=zeros(Nu2,1); %allocate memory to speed up for loop
for i=1:Nu2
    t(i,1)=to+(i-1)*ts; %time vector [s]
end
t=Ts.*round(t./Ts); 
%round elements of time vector to the nearest sampling interval

%Output Structure
%----------------
out.samplingTime=Ts;
out.decimationFactor=decRecData;
out.stepSequenze=[t u2 r1];
out.nrOfSteps=Nu2;