%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul
% Date: 31. Januar 2012
% Revision: -
%**************************************************************************

function [out]=constParamDistFreqRespFlexShaft(Ts)
%Input:
%  Ts = sampling time [s]
%Output:
%  out = constant parameter
figureNr=1; %base figure number
enGraphicOut=0; %enable graphical output: 0=disable, 1=enable

O=-0.7; %-0.9; %offset reference current motor 2 [A] (for all w)

%range 1 w=0.2-0.8 [rad/s]
%-------------------------
w1=(0.2:0.3:0.8)'; %frequency reference voltage motor 1 [rad/s]
N1=length(w1);
D1=[18.0; 1.0*ones(N1-1,1)]; %delay [s]
A1=0.1*ones(N1,1); %0.1 %amplitude reference current motor 2 [A]
P1=1.0*ones(N1,1); %number of periods

%range 2 w=1.0-5.0 [rad/s]
%-------------------------
w2=(1.0:1.0:5.0)'; %frequency reference voltage motor 1 [rad/s]
N2=length(w2);
D2=[18.0; 1.0*ones(N2-1,1)]; %delay [s]
A2=0.1*ones(N2,1); %0.2 %amplitude reference current motor 2 [A]
P2=2.0*ones(N2,1); %number of periods

%range 3 w=5.8-7.8 [rad/s]
%--------------------------
w3=(5.6:0.4:7.6)'; %frequency reference voltage motor 1 [rad/s]
N3=length(w3);
D3=[18.0; 18.0*ones(N3-1,1)]; %delay [s]
A3=0.1*ones(N3,1); %0.2 %amplitude reference current motor 2 [A]
P3=3.0*ones(N3,1); %number of periods

%range 4 w=7.8-10.0 [rad/s]
%--------------------------
w4=(7.8:0.2:10.0)'; %frequency reference voltage motor 1 [rad/s]
N4=length(w4);
D4=[36.0; 18.0*ones(N4-1,1)]; %delay [s]
A4=0.1*ones(N4,1); %0.2 %amplitude reference current motor 2 [A]
P4=9.0*ones(N4,1); %number of periods

%range 5 w=9.9-12.0 [rad/s]
%--------------------------
w5=(10.4:0.4:12.0)'; %frequency reference voltage motor 1 [rad/s]
N5=length(w5);
D5=[36.0; 18.0*ones(N5-1,1)]; %delay [s]
A5=0.25*ones(N5,1); %0.5 %amplitude reference current motor 2 [A]
P5=12.0*ones(N5,1); %number of periods

%range 6 w=14.0-18.0 [rad/s]
%---------------------------
w6=(14.0:2.0:18.0)'; %frequency reference voltage motor 1 [rad/s]
N6=length(w6);
D6=[36.0; 18.0*ones(N6-1,1)]; %delay [s]
A6=0.4*ones(N6,1); %0.8 %amplitude reference current motor 2 [A]
P6=15.0*ones(N6,1); %number of periods

% %range 7 w=20.0-30.0 [rad/s]
% %---------------------------
% w7=(20.0:10.0:30.0)'; %frequency reference voltage motor 1 [rad/s]
% N7=length(w7);
% D7=[36.0; 36.0*ones(N7-1,1)]; %delay [s]
% A7=0.6*ones(N7,1); %1.3 %amplitude reference current motor 2 [A]
% P7=18.0*ones(N7,1); %number of periods
% 
% 
% w=[w1; w2; w3; w4; w5; w6; w7];
% N=length(w); %number of frequencies
% D=[D1; D2; D3; D4; D5; D6; D7];
% P=[P1; P2; P3; P4; P5; P6; P7];
% A=[A1; A2; A3; A4; A5; A6; A7];


w=[w1; w2; w3; w4; w5; w6];
N=length(w); %number of frequencies
D=[D1; D2; D3; D4; D5; D6];
P=[P1; P2; P3; P4; P5; P6];
A=[A1; A2; A3; A4; A5; A6];


ws=2*pi/Ts; %sampling frequency [rad/s]
%preallocate memory to speed up for loop
Mf=zeros(N,1); %number of samples freq
Md=zeros(N,1); %number of samples delay
phi=zeros(N,1); %phases
ak=zeros(N,1); %Goertzel filter coefficients
for i=1:N
    Mhp=round(ws/(2*w(i,1))); %number of samples one half period
    w(i,1)=ws/(2*Mhp); % => w*Mhp*Ts=pi or w*Mhp*2=2*pi
    Md(i,1)=ceil(floor(D(i,1)/Ts)/Mhp)*Mhp-1; %number of samples delay
    %=> w*Md*Ts=k*pi-w*Ts, k=0,1,2,...
    Mf(i,1)=Md(i,1)+1+Mhp*2*P(i,1); %=> w*Mf*Ts=k*pi, k=0,1,2,...
    if (i>1)
        phi(i,1)=mod(w(i-1,1)*Mf(i-1,1)*Ts+phi(i-1,1),2*pi);
    else
        phi(i,1)=0;
    end
    ak(i,1)=2*cos(2*pi*w(i,1)/ws); %Goertzel filter coefficient
end

if enGraphicOut
    i=23;
    F=linspace(0.1, 100, 1000);
    H=freqz(1,[1 -ak(i,1) 1],F,1/Ts);
    figure(figureNr);
    subplot(211);
    semilogx(2*pi.*F,abs(H),'r');
    grid on;
    xlabel('frequency [rad/s]');
    ylabel('magnitude');
    title('Frequency Response Goertzel Filter');
    subplot(212);
    semilogx(2*pi.*F,angle(H)*180/pi,'r');
    grid on;
    xlabel('frequency [rad/s]');
    ylabel('phase [deg]');
end

%Output Structure
%----------------
out.samplingTime=Ts;
out.nrOfFreqs=N;
out.nrOfSamplesDelay=Md;
out.nrOfSamplesFreq=Mf;
out.freqs=w;
out.phases=phi;
out.filterCoeffs=ak;
out.amplitudes=A;
out.offset=O;