%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul
% Date: 31. Januar 2012
% Revision: -
%**************************************************************************

function [out]=constParamManipulatingFreqRespFlexShaft(Ts)
%Input:
%  Ts = sampling time [s]
%Output:
%  out = constant parameter

O=8.0; %9.0 %offset reference voltage motor 1 [V] (for all w)

%range 1 w=0.1-0.7 [rad/s]
%-------------------------
w1=(0.1:0.3:0.7)'; %frequency reference voltage motor 1 [rad/s]
N1=length(w1);
D1=[18.0; 1.0*ones(N1-1,1)]; %delay [s]
A1=1.0*ones(N1,1); %1.0 %amplitude reference voltage motor 1 [V]
P1=1.0*ones(N1,1); %number of periods

%range 2 w=1.0-3.0 [rad/s]
%-------------------------
w2=(1.0:1.0:3.0)'; %frequency reference voltage motor 1 [rad/s]
N2=length(w2);
D2=[18.0; 1.0*ones(N2-1,1)]; %delay [s]
A2=2.0*ones(N2,1); %3.0 %amplitude reference voltage motor 1 [V]
P2=2.0*ones(N2,1); %number of periods

%range 3 w=3.8-7.1 [rad/s]
%-------------------------
w3=(3.8:0.3:7.1)'; %frequency reference voltage motor 1 [rad/s]
N3=length(w3);
D3=[36.0; 18.0*ones(N3-1,1)]; %delay [s]
A3=4.0*ones(N3,1); %8.0 %amplitude reference voltage motor 1 [V]
P3=6.0*ones(N3,1); %number of periods

%range 4 w=7.4-11.9 [rad/s]
%--------------------------
w4=(7.4:0.3:11.9)'; %frequency reference voltage motor 1 [rad/s]
N4=length(w4);
D4=[54.0; 18.0*ones(N4-1,1)]; %36,18 %delay [s]
A4=1.0*ones(N4,1); %2.0 %amplitude reference voltage motor 1 [V]
P4=9.0*ones(N4,1); %number of periods

%range 5 w=12.5-17.5 [rad/s]
%---------------------------
w5=(12.5:2.5:17.5)'; %frequency reference voltage motor 1 [rad/s]
N5=length(w5);
D5=[36.0; 18.0*ones(N5-1,1)]; %36,18 delay [s]
A5=2.0*ones(N5,1); %3.5 %amplitude reference voltage motor 1 [V]
P5=15.0*ones(N5,1); %number of periods

%range 6 w=20.0-40.0 [rad/s]
%---------------------------
w6=(20.0:10.0:40.0)'; %frequency reference voltage motor 1 [rad/s]
N6=length(w6);
D6=[36.0; 36.0*ones(N6-1,1)]; %delay [s]
A6=2.0*ones(N6,1); %6.0 %amplitude reference voltage motor 1 [V]
P6=18.0*ones(N6,1); %number of periods

w=[w1; w2; w3; w4; w5; w6];
N=length(w); %number of frequencies
D=[D1; D2; D3; D4; D5; D6];
P=[P1; P2; P3; P4; P5; P6];
A=[A1; A2; A3; A4; A5; A6];

ws=2*pi/Ts; %sampling frequency [rad/s]
%preallocate memory to speed up for loop
Mf=zeros(N,1); %number of samples per frequency
Md=zeros(N,1); %number of samples per delay
phi=zeros(N,1); %phases
ak=zeros(N,1); %Goertzel filter coefficients
for i=1:N
    Mhp=round(ws/(2*w(i,1))); %number of samples one half period
    w(i,1)=ws/(2*Mhp); % => w*Mhp*Ts=pi or w*Mhp*2=2*pi
    Md(i,1)=ceil(floor(D(i,1)/Ts)/Mhp)*Mhp-1; %number of samples delay
    %=> w*Md*Ts=k*pi-w*Ts, k=0,1,2,...
    Mf(i,1)=Md(i,1)+1+Mhp*2*P(i,1); %=> w*Mf*Ts=k*pi, k=0,1,2,...
    if (i>1)
        phi(i,1)=mod(w(i-1,1)*Mf(i-1,1)*Ts+phi(i-1,1),2*pi);
    else
        phi(i,1)=0;
    end
    ak(i,1)=2*cos(2*pi*w(i,1)/ws); %Goertzel filter coefficient
end

%Output Structure
%----------------
out.samplingTime=Ts;
out.nrOfFreqs=N;
out.nrOfSamplesDelay=Md;
out.nrOfSamplesFreq=Mf;
out.freqs=w;
out.phases=phi;
out.filterCoeffs=ak;
out.amplitudes=A;
out.offset=O;