%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul (bpa)
% Date: 31. Januar 2012
% Revision: 22. Januar 2013 bpa
%**************************************************************************
%This file is automatically executed by the init callback function of
%the following two files (see model properties):
%  -simFlexShaft (indirectly via updateVarParamShaft)
%  -targetSysFlexShaft

clc;
Ts=5e-3; %sampling time [s]
decRecData=2; %decimation recording data

%--------------------------------------------------------------------------
% Mode Selector
%--------------------------------------------------------------------------
modeSel=0;

%--------------------------------------------------------------------------
% CMD
%--------------------------------------------------------------------------
cmd=0;

%--------------------------------------------------------------------------
% Function Selector
%--------------------------------------------------------------------------
fcnSel=0;

%--------------------------------------------------------------------------
% Controller Selector
%--------------------------------------------------------------------------
controllerSel=0;

%--------------------------------------------------------------------------
% State Space Model Plant
%--------------------------------------------------------------------------
J=402e-6; %default inertia left respectively right flywheel  [kgm^2/rad]
[sysc, syscr]=continuousTimeSsModelFlexShaft(J);
% sysc = continuous time state space model
% syscr = continuous time reduced state space model
sysMatrixAcPlant=sysc.Ac;
sysMatrixBcPlant=sysc.Bc;
sysMatrixCcPlant=sysc.Cc;
sysMatrixDcPlant=sysc.Dc;
clear sysc

%--------------------------------------------------------------------------
% PI Feedback Controller
%--------------------------------------------------------------------------
integralGainPidController=0.0;
proportionalGainPidController=0.0;

%--------------------------------------------------------------------------
% Loop Shaping Controller
%-------------------------------------------------------------------------- 
numzTfLoopShapingController=1e-3*ones(1,4);
denzTfLoopShapingController=[1 1e-3*ones(1,3)];

%--------------------------------------------------------------------------
% State Feedback Controller
%--------------------------------------------------------------------------

%Discrete Time State Space Model
%-------------------------------
sysd=c2d(ss(syscr.Ac,syscr.Bc,syscr.Cc,syscr.Dc),Ts,'zoh');
%  d/dtx(t)=Ac*x(t)+Bc*w(t), y=Cc*x(t)+Dc*w(t)
%  x(t)=[x1(t) ... xn(t)]', w(t)=[u(t) z(t)]', y(t)=x1(t)
%  x(k+1)=Ad*x(k)+Bd*w(k), y(k)=Cd*x(k)+Dd*w[k],
%  x(k)=[x1(k) ... xn(k)]', w(k)=[u(k) z(k)]', y(k)=x1(k)
%  Ad=sysd.a, Bd=sysd.b, Cd=sysd.c, Dd=sysd.d
sysMatrixAdObserver=sysd.a; 
sysMatrixBduObserver=sysd.b(:,1);
sysMatrixCdObserver=sysd.c;
[N,M]=size(sysd.b); %N=number of states; M=number of inputs
clear sysd

%Feedback Gain State Controller
%------------------------------
integralGainStateController=0.0;
feedbackGainStateController=1e-3*ones(1,N);

%Observer Gain
%-------------
feedbackGainObserver=1e-3*ones(N,1);
clear N M

%--------------------------------------------------------------------------
% Output Structure Constant Parameter
%--------------------------------------------------------------------------
constParamFlexShaft.samplingTime=Ts;
cd('ConstantParameterFlexibleShaft');
constParamFlexShaft.interfaceCoder1=...
    constParamInterfaceCoder1FlexShaft();
constParamFlexShaft.interfaceCoder2=...
    constParamInterfaceCoder2FlexShaft();
constParamFlexShaft.interfacePowerAmpMot12=...
    constParamInterfacePowerAmpMot12FlexShaft();
constParamFlexShaft.control=...
    constParamControlFlexShaft(Ts,decRecData);
cd ..

