%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul (bpa)
% Date: 20. Februar 2012
% Revision: 22. Januar 2013 (bpa)
%**************************************************************************
%This file is automatically executed by the stop callback function of the
%following two files (see model properties):
%  -simFlexShaft
%  -targetSysFlexShaft
%The following workspace variables will be used by this m-file:
%  -constParamFlexShaft
%  -syscr
%  -recData*

if ~exist('BATCH_REDO_FIGS','var')
clearvars -except constParamFlexShaft syscr recData* ...
         fcnSel controllerSel modeSel J
end
%close all;
%clc;
figureNr=1; %base figure number
doAutosaveRecdata=1; %enable autosaving of recorded data
enTfModel=0; %enable transfer function model: 0=disable, 1=enable
if (~exist('constParamFlexShaft','var'));
    error('variable constParamFlexShaft not found in workspace');
end
Ts=constParamFlexShaft.samplingTime; %sampling time [s]
ws=2*pi/Ts; %sampling frequency [rad/s];
if (~exist('syscr','var')); %continuous time reduced state space model
    error('variable syscr not found in workspace');
end
sysc_yu=ss(syscr.Ac,syscr.Bc(:,1),syscr.Cc,syscr.Dc(1,1)); 
sysc_yz=ss(syscr.Ac,syscr.Bc(:,2),syscr.Cc,syscr.Dc(1,2));
nshown=0;
%--------------------------------------------------------------------------
% Recorded Data Manipulating Frequency Response
%--------------------------------------------------------------------------
if (exist('recDataManipulatingFreqResp','var'))
    if (~isempty(recDataManipulatingFreqResp))
        nshown=nshown+1;
        y_n  =recDataManipulatingFreqResp(:,1);
        y_nm1=recDataManipulatingFreqResp(:,2);
        u_n  =recDataManipulatingFreqResp(:,3);
        u_nm1=recDataManipulatingFreqResp(:,4);
        w    =recDataManipulatingFreqResp(:,5);
        Mm   =recDataManipulatingFreqResp(:,6);
        N=length(w); %number of frequencies
        j=sqrt(-1);
        %preallocate memory to speed up for loop
        G=zeros(N,1);
        %do post calculation
        for i=1:N
            Y = (y_n(i,1)-exp(-j*2*pi*w(i,1)/ws)*y_nm1(i,1))/Mm(i,1);
            U = (u_n(i,1)-exp(-j*2*pi*w(i,1)/ws)*u_nm1(i,1))/Mm(i,1);
            G(i,1) = Y/U;
        end
        measuredMag=abs(G);
        measuredPhase=unwrap(angle(G));
        
        [ssModelMag ssModelPhase]=bode(sysc_yu,w);
        ssModelMag=squeeze(ssModelMag(1,1,:));
        ssModelPhase=squeeze(ssModelPhase(1,1,:));
        
        if enTfModel
            [numz,denz]=freqResp2TfModel(G,w,Ts,2,4);
            [tfModelMag tfModelPhase]=dbode(numz,denz,Ts,w);
            [p,z]=pzmap(numz,denz);
            disp('poles discrete time transfer function model');
            disp(p);
            disp('zeros discrete time transfer function model');
            disp(z);
            title('pole zero map transfer funtion model G(s)=Y(s)/U(s)');
        end
        
        figure(figureNr);
        figureNr=figureNr+1;
        subplot(211);
        semilogx(w,20*log10(measuredMag),'r');
        hold on;
        semilogx(w,20*log10(ssModelMag),'b');
        if enTfModel
            semilogx(w,20*log10(tfModelMag),'g');
        end
        hold off;
        grid on;
        xlabel('frequency [rad/s]');
        ylabel('magnitude [dB]');
        if enTfModel
            legend('Measured','State Space Model',...
                'Transfer Function Model','location','southwest');
        else
            legend('Measured','State Space Model',...
                'location','southwest');
        end
        titlePart1='Recorded Data Input-Output Frequency Response: ';
        titlePart2='G(s)=Y(s)/U(s)';
        title([titlePart1 titlePart2]);
        
        subplot(212);
        semilogx(w,measuredPhase*180/pi,'r');
        hold on;
        semilogx(w,ssModelPhase,'b');
        if enTfModel
            semilogx(w,tfModelPhase,'g');
        end
        hold off;
        grid on;
        xlabel('frequency [rad/s]');
        ylabel('phase [degree]');
    end
end

%--------------------------------------------------------------------------
% Recorded Data Disturbance Frequency Response
%--------------------------------------------------------------------------
if (exist('recDataDistFreqResp','var'))
    if (~isempty(recDataDistFreqResp))  
        nshown=nshown+1;
        y_n  =recDataDistFreqResp(:,1);
        y_nm1=recDataDistFreqResp(:,2);
        u_n  =recDataDistFreqResp(:,3);
        u_nm1=recDataDistFreqResp(:,4);
        w    =recDataDistFreqResp(:,5);
        Mm   =recDataDistFreqResp(:,6);
        N=length(w); %number of frequencies
        j=sqrt(-1);
        %preallocate memory to speed up for loop
        G=zeros(N,1);
        %do post calculation
        for i=1:N
            Y = (y_n(i,1)-exp(-j*2*pi*w(i,1)/ws)*y_nm1(i,1))/Mm(i,1);
            U = (u_n(i,1)-exp(-j*2*pi*w(i,1)/ws)*u_nm1(i,1))/Mm(i,1);
            G(i,1) = Y/U;
        end
        measuredMag=abs(G);
        measuredPhase=unwrap(angle(G));
        
        [ssModelMag ssModelPhase]=bode(sysc_yz,w);
        ssModelMag=squeeze(ssModelMag(1,1,:));
        ssModelPhase=squeeze(ssModelPhase(1,1,:));
        
        if enTfModel
            [numz,denz]=freqResp2TfModel(G,w,Ts,0,4);
            [tfModelMag tfModelPhase]=dbode(numz,denz,Ts,w);
            [p,z]=pzmap(numz,denz);
            disp('poles discrete time transfer function model');
            disp(p);
            disp('zeros discrete time transfer function model');
            disp(z);
        end
        
        figure(figureNr);
        figureNr=figureNr+1;
        subplot(211);
        semilogx(w,20*log10(measuredMag),'r');
        hold on;
        semilogx(w,20*log10(ssModelMag),'b');
        if enTfModel
            semilogx(w,20*log10(tfModelMag),'g');
        end
        hold off;
        grid on;
        xlabel('frequency [rad/s]');
        ylabel('magnitude [dB]');
        if enTfModel
            legend('Measured','State Space Model',...
                'Transfer Function Model','location','southwest');
        else
            legend('Measured','State Space Model',...
                'location','southwest');
        end
        titlePart1='Recorded Data Disturbance Frequency Response: ';
        titlePart2='Gz(s)=Y(s)/Z(s)';
        title([titlePart1 titlePart2]);
        
        subplot(212);
        semilogx(w,measuredPhase*180/pi,'r');
        hold on;
        semilogx(w,ssModelPhase,'b');
        if enTfModel
            semilogx(w,tfModelPhase,'g');
        end
        hold off;
        grid on;
        xlabel('frequency [rad/s]');
        ylabel('phase [degree]');
    end
end

%--------------------------------------------------------------------------
% Recorded Data Manipulating Step Response
%--------------------------------------------------------------------------
if (exist('recDataManipulatingStepResp','var'))
    if (~isempty(recDataManipulatingStepResp))
        nshown=nshown+1;
        refVoltageMot1  =recDataManipulatingStepResp(:,1);
        estimatedVelMot1=recDataManipulatingStepResp(:,2);
        time            =recDataManipulatingStepResp(:,3);
        
        ssModelEstimatedVelMot1=lsim(sysc_yu,refVoltageMot1,time);
        
        figure(figureNr);
        figureNr=figureNr+1;
        subplot(211);
        stairs(time,refVoltageMot1,'b');
        grid on;
        xlabel('Time [s]');
        %ylabel('Reference Voltage Motor 1 [V]');
        ylabel({'u','Voltage Motor 1 [V]'});        
        title('Recorded Data Input-Output Step Response');
        
        subplot(212);
        stairs(time,estimatedVelMot1,'r');
        hold on;
        stairs(time,ssModelEstimatedVelMot1,'b');
        hold off;
        grid on;
        xlabel('Time [s]');
        %ylabel('Velocity Motor 1 [rad/s]');
        ylabel({'y','Velocity Motor 1 [rad/s]'});        
        legend('Measured', 'State Space Model');
    end
end

%--------------------------------------------------------------------------
% Recorded Data Open-Loop Disturbance Step Response
%--------------------------------------------------------------------------
if (exist('recDataOpenLoopDistStepResp','var'))
    if (~isempty(recDataOpenLoopDistStepResp))
        nshown=nshown+1;
        refCurrentMot2  =recDataOpenLoopDistStepResp(:,1);
        estimatedVelMot1=recDataOpenLoopDistStepResp(:,2);
        time            =recDataOpenLoopDistStepResp(:,3);
        
        ssModelEstimatedVelMot1=lsim(sysc_yz,refCurrentMot2,time);
        
        figure(figureNr);
        figureNr=figureNr+1;
        subplot(211);
        stairs(time,refCurrentMot2,'b');
        grid on;
        xlabel('Time [s]');
        %ylabel('Reference Current Motor 2 [A]');
        ylabel({'z';'Current Motor 2 [A]'});        
        title('Recorded Data Disturbance Step Response');
        
        subplot(212);
        stairs(time,estimatedVelMot1,'r');
        hold on;
        stairs(time,ssModelEstimatedVelMot1,'b');
        hold off;
        grid on;
        xlabel('Time [s]');
        %ylabel('Velocity Motor 1 [rad/s]');
        ylabel({'y';'Velocity Motor 1 [rad/s]'});        
        legend('Measured', 'State Space Model');
    end
end

%--------------------------------------------------------------------------
% Recorded Data Reference Step Response
%--------------------------------------------------------------------------
if (exist('recDataRefStepResp','var'))
    if (~isempty(recDataRefStepResp))
        nshown=nshown+1;
        refVoltageMot1  =recDataRefStepResp(:,1);
        estimatedVelMot1=recDataRefStepResp(:,2);
        refVelMot1      =recDataRefStepResp(:,3);
        time            =recDataRefStepResp(:,4);
        
        figure(figureNr);
        figureNr=figureNr+1;
        subplot(211);
        stairs(time,refVoltageMot1,'b');
        grid on;
        xlabel('Time [s]');
        %ylabel('Reference Voltage Motor 1 [V]');
        ylabel({'u','Voltage Motor 1 [V]'});       
        title('Recorded Data Reference Step Response');
        
        subplot(212);
        stairs(time,refVelMot1,'b');
        hold on;
        stairs(time,estimatedVelMot1,'r');
        hold off;
        grid on;
        xlabel('Time [s]');
        %ylabel('Velocity Motor 1 [rad/s]');
        ylabel({'r, y' ,'Velocity Motor 1 [rad/s]'});      
        legend('Reference', 'Measured');
    end
end

%--------------------------------------------------------------------------
% Recorded Data Closed-Loop Disturbance Step Response
%--------------------------------------------------------------------------
if (exist('recDataClosedLoopDistStepResp','var'))
    if (~isempty(recDataClosedLoopDistStepResp))
        nshown=nshown+1;
        refVoltageMot1  =recDataClosedLoopDistStepResp(:,1);
        estimatedVelMot1=recDataClosedLoopDistStepResp(:,2);
        refVelMot1      =recDataClosedLoopDistStepResp(:,3);
        refCurrentMot2  =recDataClosedLoopDistStepResp(:,4);
        time            =recDataClosedLoopDistStepResp(:,5);
        
        figure(figureNr);
        figureNr=figureNr+1;
        subplot(311);
        stairs(time,refVoltageMot1,'b');
        grid on;
        xlabel('Time [s]');
        %ylabel({'Reference Voltage';'Motor 1 [V]'});
        ylabel({'u','Voltage Mot1 [V]'});
        title('Recorded Data Closed-Loop Disturbance Step Response');
        
        subplot(312);
        stairs(time,refCurrentMot2,'b');
        grid on;
        xlabel('Time [s]');
        %ylabel({'Reference Current';'Motor 2 [A]'})
        ylabel({'z' ,'Current Mot2 [A]'});
        
        subplot(313);
        stairs(time,refVelMot1,'b');
        hold on;
        stairs(time,estimatedVelMot1,'r');
        hold off;
        grid on;
        xlabel('Time [s]');
        %ylabel({'Velocity';'Motor 1 [rad/s]'});
        ylabel({'r, y' ,'Velocity Mot1 [rad/s]'});
        legend('Reference', 'Measured');
    end
end



if ~exist('BATCH_REDO_FIGS','var')   

    if doAutosaveRecdata
        if nshown>1
            error('inconsistent');
        elseif nshown==1

            switch modeSel
                case 0
                    m='sim_';
                case 1
                    m='real_';
                otherwise
                    error('unknown modeSel');
            end
            switch fcnSel
                case 0
                    f='bode_manip';
                case 1
                    f='bode_dist';
                case 2
                    f='ol_manip';
                case 3
                    f='ol_dist';
                case 4
                    f='cl_ref';
                case 5
                    f='cl_dist';
                otherwise
                    error('unknown fcnSel');
            end
            if fcnSel>=4 
                switch controllerSel
                    case 0
                        ctlInfo='_pi';
                    case 1
                        ctlInfo='_ls';
                    case 2
                        ctlInfo='_sfb';
                    otherwise
                        error('unknown controllerSel');
                end
            else
                ctlInfo='';
            end
            switch J
                case 402e-6
                    Jinfo='';
                case 1002e-6
                    Jinfo='_incrJ';
                otherwise
                    error('unknown J');
            end
            name = [m,f,ctlInfo,Jinfo]; 
            outdir='out';
            [suc,msg,msgid]=mkdir(outdir); %outputs to suppress warning
            save(fullfile(outdir,name),'constParamFlexShaft','syscr','recData*','fcnSel','controllerSel','modeSel','J');
        end
        
    end
    
    
clearvars -except constParamFlexShaft syscr recData* ... %clean up workspace
     fcnSel controllerSel modeSel J    

end


