%**************************************************************************
% Automatic Control Laboratory 
% Brunner Paul (bpa)
% Date: 17. Januar 2013
% Revision: 
%**************************************************************************
% Template for students for FlexShaft A/B
%**************************************************************************

initFcnFlexShaft();

%==========================================================================
% CONFIGURATION (configure it according to given task)
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% Mode Selector
%--------------------------------------------------------------------------
modeSel=0; %0...1
%  0=simulator
%  1=target

%--------------------------------------------------------------------------
% CMD
%--------------------------------------------------------------------------
cmd=1; %0 ... 1
%  1=run
%  0=reset

%--------------------------------------------------------------------------
% Function Selector
%--------------------------------------------------------------------------
fcnSel=2; %0 ... 5
%  0=measure input-output frequency response
%    G(s)=Y(s)/U(s)
%  1=measure disturbance frequency response
%    Gz(s)=Y/(s)/Z(s)
%
%  2=measure input-output step response
%    u(t)=step sequenze; z(t)=0
%  3=measure disturbance step response
%    u(t)=0; z(t)=step sequenze 
%
%  4=measure reference step response
%    r(t)=step sequenze; z(t)=0
%  5=measure closed-loop disturbance step response
%    r(t)=three levels; z(t)=step sequenze

%--------------------------------------------------------------------------
% Controller Selector (for fncSel==4 or fcnSel==5)
%--------------------------------------------------------------------------
controllerSel=0; % 0 ... 2,
%  0=pid
%  1=loopShaping  %only for part B!
%  2=state        %only for part B!

%--------------------------------------------------------------------------
% Inertia Flywheel
%--------------------------------------------------------------------------
J=402e-6; %inertia of each flywheel  [kgm^2/rad]
%J=1002e-6; %increased inertia of each flywheel  [kgm^2/rad]

%==========================================================================

% [do not change]

validateattributes(modeSel,{'double'},{'binary'},'','modeSel');
validateattributes(cmd,{'double'},{'binary'},'','cmd');
validateattributes(fcnSel,{'double'},{'integer','>=',0,'<=',5}, ...
    '','fcnSel');
validateattributes(controllerSel,{'double'},{'integer','>=',0,'<=',2}, ...
    '','controllerSel');

Ts=constParamFlexShaft.samplingTime; %sampling time [s]

[sysc, syscr]=continuousTimeSsModelFlexShaft(J);
sysMatrixAcPlant=sysc.Ac;
sysMatrixBcPlant=sysc.Bc;
sysMatrixCcPlant=sysc.Cc;
sysMatrixDcPlant=sysc.Dc;
clear sysc


%==========================================================================
%% MODEL ANALYSIS and CONTROLLER DESIGN
%--------------------------------------------------------------------------

%% README:
%-write your code in this section!
%-the continuous-time state-space model is available in struct syscr
%-the section numbers and tasks are according to the preparation@home parts of the manual
%-in this script the variable names for the continuous-time transfer functions
% are written with an additional c at the end of the name (e.g. Gc instead of G)


%--------------------------------------------------------------------------
%% A-2.2 Analysis of the Flexible Shaft Model
%--------------------------------------------------------------------------

%%--- Task A.1 ---

%use: syscr.Ac,syscr.Bc,syscr.Cc,syscr.Dc


%input-output transfer function Gc(s)=Y(s)/U(s)
%Gc=
%disturbance transfer function Gzc(s)=Y(s)/Z(s)
%Gzc=


%%--- Task A.2 ---

%Bode plot of Gc(s)

%Bode plot of Gzc(s)



%--- Task A.4 ---

%disp('poles Gc(s)=Y(s)/U(s):');

%disp('zeros Gc(s)=Y(s)/U(s):');

%disp('zeros Gzc(s)=Y(s)/Z(s):');





%--------------------------------------------------------------------------
%% A-2.3 PI Controller
%--------------------------------------------------------------------------

%--- Task A.7 ---


%>> you have to find the PI parameters
Ki=0; %to change
Kp=0; %to change


%Bode plot of Lc(s)=Grc(s)*Gc(s)
 




%Output [do not change]
%------
integralGainPidController=Ki;
proportionalGainPidController=Kp;





%--------------------------------------------------------------------------
%% B-2.2 Loop Shaping Controller
%-------------------------------------------------------------------------- 

%--- Task B.1 ---

%>> you have to define Grc, the continuous-time transfer function of the controller.
%Note: the hightest possible order for numerator or denominator is four!
Grc=tf([1],[1]); %to change


%Bode plot of Lc(s)=Grc(s)*Gc(s)



%Output [do not change]
%------
Grd=c2d(Grc,Ts,'tustin'); %discrete time
assert(length(Grd.num{1,1})<=4,'order of numerator must be <=4');
assert(length(Grd.den{1,1})<=4,'order of denominator must be <=4');
numzTfLoopShapingController=Grd.num{1,1};
denzTfLoopShapingController=Grd.den{1,1};




%--------------------------------------------------------------------------
%% B-2.3 State Feedback Controller
%--------------------------------------------------------------------------
nx=size(syscr.Bc,1); %nx=number of states
%--------------------------------------------------------------------------

%--- Task B.2 --- 
%Discrete-Time State Space Model
%-------------------------------

%Note: the continuous-time matrices are available in struct syscr and
% the sampling time in variable Ts

%>> finally, you have to set the matrices of the discrete-time state-space model
Ad=zeros(size(syscr.Ac)); %to change
Bd=zeros(size(syscr.Bc)); %to change
Cd=zeros(size(syscr.Cc)); %to change
Dd=zeros(size(syscr.Dc)); %to change

%disp('eigenvalues matrix Ad:');

%disp('eigenvalues of Ac transformed into the z-domain:');

%disp('controllability (Ad,Bdu):');

%disp('observability (Ad,Cd):');




%--- Task B.3 ---
%Full-State Feedback Controller
%-------------------------------

%extended system with integral action (integration method: forward euler)
%  x_bar(k+1)=Ad_bar*x_bar(k)+Bd_bar*w_bar(k)  
%  y(k)      =Cd_bar*x_bar(k)+Dd_bar*w_bar(k)    
%where
%  x_bar(k)=[xI(k) x(k)]',  w_bar(k)=[u(k) r(k) z(k)]'
%Ad_bar=
%Bd_bar=
%Cd_bar=
%Dd_bar=


%disp('controllability {Ad_bar,Bdu_bar}:');


%>> you have to find the state feedback gain K_bar=[Ki K1 ... Kn]
K_bar=zeros(1,1+nx); %to change; 





%--- Task B.4 ---
%Observer Gain
%----------------


%>> you have to find the observer gain L=[L1; ...; Ln]
L=zeros(nx,1);  %to change





%--- Task B.5 ---
%Closed-Loop System With Observer
%--------------------------------
%  x_tilde(k+1)=Ad_tildecl*x_tilde(k)+Bd_tildecl*w_tildecl(k) 
%  y(k)        =Cd_tildecl*x_tilde(k)+Dd_tildecl*w_tildecl(k)
%where
%  x_tilde(k)  =[xI(k) x(k) x_hut(k)]',  w_tildecl(k)=[r(k) z(k)]'
%Ad_tildecl=
%Bd_tildecl=
%Cd_tildecl=
%Dd_tildecl=


%for the following plots, use function dbode with matrices Ad_tildecl, etc
%Bode plot of T(z)=Y(z)/R(z) (reference transfer function)

%Bode plot of Tz(z)=Y(z)/Z(z) (closed-loop disturbance transfer function)



%Output [do not change]
%------
integralGainStateController=K_bar(1,1);
feedbackGainStateController=K_bar(1,2:end);
feedbackGainObserver=L;
sysMatrixAdObserver=Ad;
sysMatrixBduObserver=Bd(:,1);
sysMatrixCdObserver=Cd;

%==========================================================================


%%
%--------------------------------------------------------------------------
% update variable parameters
%--------------------------------------------------------------------------
if (modeSel==1)
    modelName='targetSysFlexShaft';
    set_param(modelName,'SimulationCommand','update');
end


%--------------------------------------------------------------------------
% clean workspace
%--------------------------------------------------------------------------
clearvars -except cmd fcnSel controllerSel modeSel J modelName ...
    sysMatrixAcPlant sysMatrixBcPlant sysMatrixCcPlant sysMatrixDcPlant ...
    integralGainPidController proportionalGainPidController ...
    numzTfLoopShapingController denzTfLoopShapingController ...
    sysMatrixAdObserver sysMatrixBduObserver sysMatrixCdObserver ...
    integralGainStateController feedbackGainStateController ...
    feedbackGainObserver constParamFlexShaft syscr
